use bitflags::bitflags;
use bitflags_serde_shim::impl_serde_for_bitflags;
use quickpeep_densedoc::DenseDocument;
use serde::{Deserialize, Serialize};
use std::collections::BTreeSet;

bitflags! {
    pub struct AnalysisAntifeatures: u8 {
        /// Adverts are present on the page, according to a filter.
        const ADVERTS = 0x01;
        /// Some things are blocked due to privacy concerns, according to a filter.
        const PRIVACY = 0x02;
        /// Annoying cookie nags are present on this page, according to a cosmetic filter.
        const COOKIE_NAG = 0x04;
        /// Unspecified annoyances are present on this page, according to a cosmetic filter.
        const ANNOYANCE = 0x08;

        /// The web page was served over CloudFlare at the time of indexing, which is not in the
        /// spirit of decentralisation.
        const CLOUDFLARE = 0x10;
    }
}

impl_serde_for_bitflags!(AnalysisAntifeatures);

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct RakedPageEntry {
    pub analysed_antifeatures: AnalysisAntifeatures,
    pub document: DenseDocument,
}

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct RakedReferrerEntry {
    pub references: BTreeSet<RakedReference>,
}

#[derive(Serialize, Deserialize, Debug, Clone, Ord, PartialOrd, Eq, PartialEq)]
pub struct RakedReference {
    pub target: String,
    pub kind: ReferenceKind,
}

#[derive(Serialize, Deserialize, Debug, Copy, Clone, Ord, PartialOrd, Eq, PartialEq)]
pub enum ReferenceKind {
    /// Canonical URL for the same document, as declared in the page.
    CanonicalUrl,
    /// HTTP-level redirect.
    Redirect,
    /// Link in a page (<a>). Could be to another page or to a feed.
    Link,
    /// <link> to a feed
    HeaderLinkedFeed,
    FeedEntry,
    SitemapEntry,
}
